/***************************************************************************
                          dcconfig.cpp  -  description
                             -------------------
    begin                : Fri Mar 1 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qdir.h>
#include <qstringlist.h>
#include <qregexp.h>

#include <dclib/che3.h>
#include <dclib/csocket.h>

// icons
#include "icon_operator.xpm"
#include "icon_user.xpm"
#include "icon_user_away.xpm"

#ifdef WIN32
#include <malloc.h>
#endif

#include "dcconfig.h"

// config files
#define DCGUI_CONFIG		"/.dc/dcgui.cfg"

#define XML_DCGUI_CONFIG	"dcgui"

#define XML_GUI			"gui"
#define XML_OPENPRIVATECHATWIN	"openprivatechatwindow"
#define XML_AUTOSIZECONVERT	"autosizeconvert"
#define XML_THEME		"theme"
#define XML_LANGUAGEFILE	"languagefile"
#define XML_APP_FONT		"appfont"
#define XML_APP_WIDTH		"appwidth"
#define XML_APP_HEIGHT		"appheight"
#define XML_FALSE		"false"
#define XML_TRUE		"true"

DCConfig * Config;

DCConfig::DCConfig()
{
	QDir d;
	d.mkdir(QDir::homeDirPath() + "/.dc");

	// gui defaults
	bOpenPrivateChatWindow = TRUE;
	bAutoSizeConvert       = FALSE;
	sTheme                 = "Default";
	sLanguageFile          = "";
	sAppFont               = DEFAULT_APP_FONT;
	iAppHeight             = 480;
	iAppWidth              = 640;

	icon_operator = QPixmap(icon_operator_xpm);
	icon_user = QPixmap(icon_user_xpm);
	icon_user_away = QPixmap(icon_user_away_xpm);
}

DCConfig::~DCConfig()
{
}

/** */
void DCConfig::ParseDCGuiConfig( xmlNodePtr node )
{
	xmlNodePtr n1,n2;
	xmlChar * c;

	for(n1=node;n1!=0;n1=n1->next)
	{
		/* gui entrys */
		if ( !xmlStrcmp( n1->name, (const xmlChar*)XML_GUI ) )
		{
			for(n2=n1->xmlChildrenNode;n2!=0;n2=n2->next)
			{
				if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_OPENPRIVATECHATWIN ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						if ( !xmlStrcmp( c, (const xmlChar*)XML_TRUE ) )
							bOpenPrivateChatWindow = TRUE;
						else
							bOpenPrivateChatWindow = FALSE;
						xmlFree(c);
					}
				}
				else if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_AUTOSIZECONVERT ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						if ( !xmlStrcmp( c, (const xmlChar*)XML_TRUE ) )
							bAutoSizeConvert = TRUE;
						else
							bAutoSizeConvert = FALSE;
						xmlFree(c);
					}
				}
				else if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_THEME ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						sTheme = FromUtf8((char*)c);
						xmlFree(c);
					}
				}
				else if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_LANGUAGEFILE ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						sLanguageFile = FromUtf8((char*)c);
						xmlFree(c);
					}
				}
				else if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_APP_FONT ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						sAppFont = FromUtf8((char*)c);
						xmlFree(c);
					}
				}
				else if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_APP_WIDTH ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						iAppWidth = atoi((char*)c);
						xmlFree(c);
					}
				}
				else if ( !xmlStrcmp( n2->name, (const xmlChar*)XML_APP_HEIGHT ) )
				{
					if ( (c = xmlNodeGetContent(n2)) != 0 )
					{
						iAppHeight = atoi((char*)c);
						xmlFree(c);
					}
				}
			}
		}
	}
}

/** */
int DCConfig::Load()
{
	LoadDCLib();
	return LoadDCGui();
}

/** */
int DCConfig::Save()
{
	SaveDCLib();
	return SaveDCGui();
}

/** */
int DCConfig::LoadDCGui()
{
	CString s;
	xmlDocPtr doc;
	xmlNodePtr node;

	s = (QDir::homeDirPath() + DCGUI_CONFIG).ascii();
	doc = xmlParseFile(s.Data());

	if ( doc == 0 )
	{
		return -1;
	}
	else
	{
		for(node=doc->children;node!=0;node=node->next)
		{
			if ( !xmlStrcmp( node->name, (const xmlChar*)XML_DCGUI_CONFIG ) )
			{
				ParseDCGuiConfig(node->xmlChildrenNode);
			}
		}

		xmlFreeDoc(doc);
	}

	return 0;
}

/** */
int DCConfig::SaveDCGui()
{
	int err=0;
	CString s;
	xmlDocPtr doc;
	xmlNodePtr node;

	doc = xmlNewDoc((const xmlChar*)"1.0");

	doc->children = xmlNewDocNode(doc,0,(const xmlChar*)XML_DCGUI_CONFIG,0);

	// gui
	node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_GUI, 0 );

	if (bOpenPrivateChatWindow==TRUE)
		s = XML_TRUE;
	else
		s = XML_FALSE;

	xmlNewChild( node, 0, (const xmlChar*)XML_OPENPRIVATECHATWIN, (const xmlChar*)(const char*)s.Data() );

	if (bAutoSizeConvert==TRUE)
		s = XML_TRUE;
	else
		s = XML_FALSE;

	xmlNewChild( node, 0, (const xmlChar*)XML_AUTOSIZECONVERT, (const xmlChar*)(const char*)s.Data() );

	xmlNewChild( node, 0, (const xmlChar*)XML_THEME, (const xmlChar*) ToUTF8(sTheme).Data() );
	xmlNewChild( node, 0, (const xmlChar*)XML_LANGUAGEFILE, (const xmlChar*) ToUTF8(sLanguageFile).Data() );
	xmlNewChild( node, 0, (const xmlChar*)XML_APP_FONT, (const xmlChar*) ToUTF8(sAppFont).Data() );
	xmlNewChild( node, 0, (const xmlChar*)XML_APP_WIDTH, (const xmlChar*) QString().setNum(iAppWidth).ascii() );
	xmlNewChild( node, 0, (const xmlChar*)XML_APP_HEIGHT, (const xmlChar*) QString().setNum(iAppHeight).ascii() );

	// save file
	s = QDir::homeDirPath() + DCGUI_CONFIG;

	if ( xmlSaveFile(s.Data(),doc) == -1 )
	{
		err = -1;
	}

	xmlFreeDoc(doc);

	return err;
}

/** */
QPixmap & DCConfig::GetUserIcon( eUserIconType type )
{
	switch(type)
	{
		case euitOPERATOR:
			return icon_operator;
			break;
		case euitUSER_AWAY:
			return icon_user_away;
			break;
		case euitUSER:
		default:
			return icon_user;
			break;
	}
}
