/***************************************************************************
                          dctransferview.cpp  -  description
                             -------------------
    begin                : Sat Feb 23 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <qstringlist.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qtextedit.h>

#include <dcconfig.h>
#include <dcbrowsefiletree.h>
#include <dcserverlist.h>
#include <dcmenuhandler.h>

#include "dctransferview.h"

DCTransferView * pTransferView = 0;

/** */
DCTransferView::DCTransferView(QWidget* parent, const char *name, int wflags) : DCDialogTransfer(parent, name, wflags)
{
	int idx;

	// set the ColumnWidthMode to manual, we will take care of this task
	for( idx = 0; idx < ListView_TRANSFER->columns(); idx++ )
	{
		ListView_TRANSFER->setColumnWidthMode( idx, QListView::Manual );
	}
	for( idx = 0; idx < ListView_TRANSFERWAIT->columns(); idx++ )
	{
		ListView_TRANSFERWAIT->setColumnWidthMode( idx, QListView::Manual );
	}
	for( idx = 0; idx < ListView_SLOTS->columns(); idx++ )
	{
		ListView_SLOTS->setColumnWidthMode( idx, QListView::Manual );
	}

	pFileList    = new CList<DCBrowseFileTree>();
	pMessageList = new CList<CObject>;

	InitDocument();
}

/** */
DCTransferView::~DCTransferView()
{
	DCBrowseFileTree * FileList;

	Timer.stop();

	Thread.Lock();

	// clear filelist
	while ( (FileList=pFileList->Next(0)) != 0 )
	{
		// remove from the list
		pFileList->Del(FileList);
	}

	delete pFileList;
	pFileList = 0;

	ListView_TRANSFERWAIT->clear();

	SaveQueue();

	Thread.UnLock();
}

/** */
void DCTransferView::SetWorkspace( QWidget* workspace )
{
	pWorkspace = (QWorkspace*)workspace;
}

/** */
void DCTransferView::InitDocument()
{
	connect( TabWidget_TRANSFER,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetCurrentChange(QWidget*)) );
	connect( ListView_TRANSFER, SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedTransferList(QListViewItem*, const QPoint &, int )) );
	connect( ListView_TRANSFERWAIT, SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedTransferWaitList(QListViewItem*, const QPoint &, int )) );

	// load download manager queue
	LoadQueue();

	connect( &Timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
	Timer.start( 500, TRUE );
}

/** */
bool DCTransferView::eventFilter( QObject * object, QEvent * event )
{
	DCBrowseFileTree * FileList;

	if((event->type() == QEvent::Close)&&((DCTransferView*)object!=this))
	{
		// file list closed
		Thread.Lock();

		if ( pFileList != 0 )
		{
			FileList = (DCBrowseFileTree *)object;
			pFileList->Remove(FileList);
		}

		Thread.UnLock();
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** current tab widget change slot */
void DCTransferView::slotTabWidgetCurrentChange(QWidget*)
{
	ResizeListViewColumn();
}

/** overridden so that the columns are resized on show() */
void DCTransferView::show()
{
	QWidget::show();

	ResizeListViewColumn();
}

/** resize event handler */
void DCTransferView::resizeEvent( QResizeEvent * )
{
	ResizeListViewColumn();
}

/** resize the ListView columns */
void DCTransferView::ResizeListViewColumn()
{
	int width;

	if ( ListView_TRANSFER->isVisible() )
	{
		width = ListView_TRANSFER->width();
	
		ListView_TRANSFER->setColumnWidth( 0, ((width*1)/10) );
		ListView_TRANSFER->setColumnWidth( 1, ((width*2)/10) );
		ListView_TRANSFER->setColumnWidth( 2, ((width*1)/10) );
		ListView_TRANSFER->setColumnWidth( 3, ((width*3)/10) );
		ListView_TRANSFER->setColumnWidth( 4, ((width*3)/10) );
	}
	if ( ListView_TRANSFERWAIT->isVisible() )
	{
		width = ListView_TRANSFERWAIT->width();

		ListView_TRANSFERWAIT->setColumnWidth( 0, ((width*4 )/24) );
		ListView_TRANSFERWAIT->setColumnWidth( 1, ((width*11)/24) );
		ListView_TRANSFERWAIT->setColumnWidth( 2, ((width*5 )/24) );
		ListView_TRANSFERWAIT->setColumnWidth( 3, ((width*4 )/24) );
	}
	if ( ListView_SLOTS->isVisible() )
	{
		width = ListView_SLOTS->width();

		ListView_SLOTS->setColumnWidth( 0, ((width*3)/9) );
		ListView_SLOTS->setColumnWidth( 1, ((width*3)/9) );
		ListView_SLOTS->setColumnWidth( 2, ((width*3)/9) );
	}
}

/** download manager callback function */
int DCTransferView::DC_DownloadManagerCallBack( CObject * Object )
{
	int err;

	Thread.Lock();

	if ( Object != NULL )
	{
		if ( pMessageList != 0 )
		{
			pMessageList->Add(Object);
			err = 0;
		}
		else
		{
			err = -1;
		}
        }
	else
	{
		err = -1;
	}

	Thread.UnLock();

	return err;
}

/** */
void DCTransferView::timerDone()
{
	CObject * Object;
	CDCMessage *DCMsg;
	bool bLVWaiUpdate = FALSE;

	for(;;)
	{
		if ( Thread.TryLock() == FALSE )
		{
			break;
		}

		if ( pMessageList != 0 )
		{
			Object = pMessageList->Next(0);

			if ( Object != 0 )
			{
				pMessageList->Remove(Object);
			}
		}
		else
		{
			Object = 0;
		}

		Thread.UnLock();

		if ( Object == 0 )
		{
			break;
		}

		DCMsg = (CDCMessage*) Object;

		switch ( DCMsg->eType )
		{
			case DC_MESSAGE_SLOT_OBJECT:
			{
				CMessageDMSlotObject * msg = (CMessageDMSlotObject *)Object;

				QListViewItem * item = ListView_SLOTS->firstChild();

				while(item)
				{
					if ( (item->text(0) == msg->sNick.Data()) &&
					     (item->text(1) == msg->sHubName.Data()) )
					{
						if ( (msg->iSlots == 0) && (!msg->bPermanent) )
						{
							ListView_SLOTS->takeItem(item);
							delete item;
							item=0;
						}

						break;
					}

					item = item->nextSibling();
				}

				if ( (!item) && ((msg->iSlots > 0) || (msg->bPermanent)) )
				{
					item = new QListViewItem( ListView_SLOTS, msg->sNick.Data(), msg->sHubName.Data() );
				}

				if (item)
				{
					if ( msg->bPermanent )
					{
						item->setText(2,"X");
					}
					else
					{
						item->setText(2,QString().setNum(msg->iSlots));
					}
				}

				break;
			}

			case DC_MESSAGE_LOG:
			{
				CMessageLog * msg = (CMessageLog*)Object;

				TextEdit_LOG->append(msg->sMessage.Data());

				break;
			}

			case DC_MESSAGE_FILE_OBJECT:
			{
				CMessageDMFileObject * msg = (CMessageDMFileObject*)Object;

				QListViewItem * item, *itemc=0;

				ListView_TRANSFERWAIT->setUpdatesEnabled(FALSE);
				bLVWaiUpdate = TRUE;

				item = ListView_TRANSFERWAIT->findItem( msg->sNick.Data(), 0 );

				if ( item == 0 )
				{
					item = new QListViewItem( ListView_TRANSFERWAIT, msg->sNick.Data() );
				}

				item->setText(1,msg->sHubName.Data());
				item->setText(2,msg->sHubHost.Data());

				switch(msg->eWaitState)
				{
					case etwsWAIT:
						item->setText(3,tr("Wait"));
						break;
					case etwsIDLE:
						item->setText(3,tr("Idle"));
						break;
					case etwsRUN:
						item->setText(3,tr("Run"));
						break;
					default:
						break;
				}

				if ( msg->sRemoteName.IsEmpty() == FALSE )
				{
					itemc = item->firstChild();

					while(itemc)
					{
						if ( itemc->text(1) == msg->sRemoteName.Data() )
						{
							break;
						}

						itemc = itemc->nextSibling();
					}

					if ( (itemc) && (msg->bRemoveFile == TRUE) )
					{
						printf("item found & remove\n");//fflush(stdout);
						item->takeItem(itemc);
						delete itemc;
					}
					else if ( msg->bRemoveFile == FALSE )
					{
						if (!itemc)
						{
							itemc = new QListViewItem( item, "" );
						}

						itemc->setText(1, msg->sRemoteName.Data());
						itemc->setText(2, CString().setNum(msg->lSize).Data());

						switch(msg->eFileState)
						{
							case etfsNONE:
								itemc->setText(3,tr("Idle"));
								break;
							case etfsTRANSFER:
								itemc->setText(3,tr("Transfer"));
								break;
							case etfsERROR:
								itemc->setText(3,tr("Error"));
								break;
							default:
								itemc->setText(3,tr("Unknown"));
								break;
       						}
					}
				}

				if ( (item->childCount() == 0) && (msg->bRemoveFile == TRUE) )
				{
					ListView_TRANSFERWAIT->takeItem(item);
					delete item;
				}

				break;
			}

			case DC_MESSAGE_TRANSFER_OBJECT:
			{
				CMessageDMTransferObject * msg = (CMessageDMTransferObject*)Object;

				QListViewItem * item;

				item = ListView_TRANSFER->firstChild();

				while(item)
				{
					if ( (item->text(4) == msg->sRemoteName.Data()) && (item->text(0) == msg->sNick.Data()) )
					{
						break;
					}
					else
					{
						item = item->nextSibling();
					}
				}

				if ( (msg->lCurrentPosition == msg->lEndPosition) || (msg->bRemoveTransfer == TRUE) )
				{
					if(item)
					{
						ListView_TRANSFER->takeItem(item);
						delete item;
					}
					break;
				}

				if ( (item == 0) && (msg->bRemoveTransfer == FALSE) )
				{
					item = new QListViewItem( ListView_TRANSFER, msg->sNick.Data(), msg->sHost.Data(), "", "", msg->sRemoteName.Data() );
				}

				switch(msg->eState)
				{
					case estTRANSFERDOWNLOAD:
						item->setText(2,tr("Download"));
						break;
					case estTRANSFERUPLOAD:
						item->setText(2,tr("Upload"));
						break;
					default:
						item->setText(2,tr("Unknown"));
						break;
				}

				ulonglong rate;
				CString r,s;

				rate = msg->lRate;
				r = "B/s";
				if ( rate >= 1024 )
				{
					r = "KB/s";
					rate/=1024;
				}

				s  = CString().setNum(msg->lCurrentPosition);
				s += "/";
				s += CString().setNum(msg->lSize);
				s += " [";
				s += CString().setNum(rate);
				s += " ";
				s += r + "]";

				rate = msg->lEndPosition-msg->lCurrentPosition;

				if ( msg->lRate>0 )
					rate/=msg->lRate;

				// This will Convert Total Secs to how much time is
				// remaining X Days + X Hrs + X Mins + X Secs
				r = " S";
				s += " [";
				if ( rate > (60*60*24) )
				{
					s+= CString().setNum(rate/(60*60*24)) +" D ";
					rate %= (60*60*24);
				}
				if ( rate > (60*60) )
				{
					s+= CString().setNum(rate/(60*60)).RightJustify(2,'0')+":";
					rate %= (60*60);
				}
				else
				{
					s+= "00:";
				}
				if ( rate > 60 )
				{
					s += CString().setNum(rate/60).RightJustify(2,'0')+":";
					rate %= 60;
				}
				else
				{
					s+= "00:";
				}
				s += CString().setNum(rate).RightJustify(2,'0').RightJustify(2,'0');
				s += "]";

				item->setText(3,s.Data());

				break;
			}

			case DC_MESSAGE_FILELIST_OBJECT:
			{
				CMessageDMFileListObject * msg = (CMessageDMFileListObject*)Object;

				NewFileBrowser(msg->sNick.Data(),msg->sHubName.Data(),msg->sHubHost.Data(),msg->sUserFileList.Data());

				break;
			}

			default:
			{
				break;
			}
		}

		delete Object;
	}

	if ( bLVWaiUpdate == TRUE )
	{
		ListView_TRANSFERWAIT->setUpdatesEnabled(TRUE);
		ListView_TRANSFERWAIT->triggerUpdate();
	}

	Timer.start( 500, TRUE );
}

/** */
void DCTransferView::NewFileBrowser( QString nick, QString hubname, QString hubhost, QString share )
{
	DCBrowseFileTree *bft;
	bft = new DCBrowseFileTree( pWorkspace, "browsefiletree", WDestructiveClose );

	// set transfer
	bft->setCaption( nick + " - Filebrowser" );
	bft->installEventFilter(this);
	bft->InitTree(nick,hubname,hubhost,share);
	pFileList->Add(bft);
	bft->show();
}

/** */
void DCTransferView::slotRightButtonClickedTransferList( QListViewItem * item , const QPoint &, int )
{
	int id;
	QPopupMenu *m;
	QString nick,host,file;

	if( item == 0 )
	{
		return;
	}

	nick = item->text(0);
	host = item->text(1);
	file = item->text(4);

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenuItem( m, emiCLOSE_TRANSFER );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == emiCLOSE_TRANSFER )
	{
		CloseTransfer( nick.ascii(), host.ascii(), file.ascii() );
	}
}

/** */
void DCTransferView::slotRightButtonClickedTransferWaitList( QListViewItem * item , const QPoint &, int )
{
	int id;
	QPopupMenu *m;
	QListViewItem * item1;
	QString nick,hubname,hubhost,file;
	CUserFileInfo UserFileInfo;

	if( item == 0 )
	{
		return;
	}

	if ( item->depth() == 1 )
	{
		item1 = item->parent();
	}
	else
	{
		item1 = item;
	}

	if ( item1 == 0 )
	{
		return;
	}

	nick    = item1->text(0);
	hubname = item1->text(1);
	hubhost = item1->text(2);

	if ( item1 != item )
	{
		file = item->text(1);
	}
	else
	{
		file = "";
	}

	if ( GetUserFileInfo(nick.ascii(),hubname.ascii(),hubhost.ascii(),file.ascii(),&UserFileInfo) == FALSE )
	{
		return;
	}

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenuItem( m, emiREMOVE_TRANSFER_QUEUE, ( (UserFileInfo.eWaitState == etwsIDLE) || ( (file != "") && (UserFileInfo.eFileState!=etfsTRANSFER))) );
	DCMenuHandler::InsertMenuItem( m, emiREMOVE_TRANSFER_QUEUE_DISK, FALSE ); //(TransferItem.eState == etwsIDLE) );
	DCMenuHandler::InsertMenuItem( m, emiSEPARATOR );
	DCMenuHandler::InsertMenuItem( m, emiTRY_CONNECT, (UserFileInfo.eWaitState == etwsIDLE) );
	DCMenuHandler::InsertMenuItem( m, emiCONNECT_TO_HUB );
	DCMenuHandler::InsertMenuItem( m, emiBROWSE_USER_FILES, (UserFileInfo.sUserFileList != "") );
	DCMenuHandler::InsertMenuItem( m, emiSEPARATOR );
	DCMenuHandler::InsertMenuItem( m, emiINFO, FALSE );
	DCMenuHandler::InsertMenuItem( m, emiSEPARATOR );
	DCMenuHandler::InsertMenuItem( m, emiSAVE_QUEUE, TRUE );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == emiCONNECT_TO_HUB )
	{
		pServerList->Connect( hubname, hubhost );
	}
	else if ( id == emiBROWSE_USER_FILES )
	{
		NewFileBrowser(nick,hubname,hubhost,UserFileInfo.sUserFileList.Data());
	}
	else if ( id == emiTRY_CONNECT )
	{
		TryConnect(nick.ascii(),hubname.ascii(),hubhost.ascii());
	}
	else if ( id == emiREMOVE_TRANSFER_QUEUE )
	{
		RemoveTransferQueue(nick.ascii(),hubname.ascii(),hubhost.ascii(),file.ascii());
	}
	else if ( id == emiSAVE_QUEUE )
	{
		SaveQueue();
	}
}
